/// Entity name which can produce multiple formats.
class EntityName {
  /// List of valid naming convention :
  /// -   snake_case
  /// -   (Camel|lower)Case
  /// -   lisp-case
  static final List<RegExp> validPatterns = <RegExp>[
    new RegExp(r'^[a-z][a-z0-9]*(_[a-z0-9]+)*$'),
    new RegExp(r'^([A-Za-z][a-z0-9]*)*$'),
    new RegExp(r'^[a-z][a-z0-9]*(-[a-z0-9]+)*$')
  ];

  /// Pattern of word splitter.
  static final RegExp splitPattern = new RegExp(r'(?=[A-Z])|_|-');

  // Segments for the name.
  // Each segment is a word in the name in its lowercase format.
  final List<String> _segments;

  /// Accepts multiple formats of the name.
  /// Currently patterns like abc_bcd, AbcBcd, abcBcd, abc-bcd are supported.
  factory EntityName(String name) {
    if (!validPatterns.any((pattern) => pattern.hasMatch(name)) ||
        name.isEmpty) {
      throw new ArgumentError(
          '$name is not valid. It should be of form "abc_bcd", '
          '"AbcBcd", "abcBcd", or "abc-bcd".');
    }

    final segments =
        name.split(splitPattern).map((s) => s.toLowerCase()).toList();
    return new EntityName._(segments);
  }

  EntityName._(this._segments);

  /// CamelCase format.
  /// Example: AbcBcdCde.
  String get camelCased =>
      _segments.map((s) => '${s[0].toUpperCase()}${s.substring(1)}').join('');

  /// lowerCamelCase format.
  /// Example: abcBcdCde.
  String get lowerCamelCased {
    var isFirst = true;
    return _segments.map((s) {
      final result = isFirst
          ? '${s[0].toLowerCase()}${s.substring(1)}'
          : '${s[0].toUpperCase()}${s.substring(1)}';
      isFirst = false;
      return result;
    }).join('');
  }

  /// snake_case format.
  /// Example: abc_bcd_cde.
  String get underscored => _segments.join('_');

  /// lisp_case format.
  /// Example: abc-bcd-cde.
  String get dashed => _segments.join('-');

  /// Space separated format.
  /// Example: Abc Bcd Cde.
  String get spaced =>
      _segments.map((s) => '${s[0].toUpperCase()}${s.substring(1)}').join(' ');
}
